@@ -28,10 +28,20 @@
 
 #include "cairo-perf.h"
 
-/* For getopt */
-#include <unistd.h>
+#include "cairo-boilerplate-getopt.h"
+
 /* For basename */
+#ifdef HAVE_LIBGEN_H
 #include <libgen.h>
+#endif
+
+#if HAVE_FCFINI
+#include <fontconfig/fontconfig.h>
+#endif
+
+#ifdef HAVE_SCHED_H
+#include <sched.h>
+#endif
 
 #define CAIRO_PERF_ITERATIONS_DEFAULT	100
 #define CAIRO_PERF_LOW_STD_DEV		0.03
@@ -65,6 +75,14 @@ target_is_measurable (cairo_boilerplate_target_t *target)
 	    return TRUE;
 	}
     case CAIRO_SURFACE_TYPE_XLIB:
+	if (strcmp (target->name, "xlib-fallback") == 0)
+	{
+	    return FALSE;
+	}
+	else
+	{
+	    return TRUE;
+	}
     case CAIRO_SURFACE_TYPE_XCB:
     case CAIRO_SURFACE_TYPE_GLITZ:
     case CAIRO_SURFACE_TYPE_QUARTZ:
@@ -84,27 +102,45 @@ target_is_measurable (cairo_boilerplate_target_t *target)
 }
 
 static const char *
-_content_to_string (cairo_content_t content)
+_content_to_string (cairo_content_t content, cairo_bool_t similar)
 {
-    switch (content) {
+    switch (content|similar) {
     case CAIRO_CONTENT_COLOR:
 	return "rgb";
+    case CAIRO_CONTENT_COLOR|1:
+	return "rgb&";
     case CAIRO_CONTENT_ALPHA:
 	return "a";
+    case CAIRO_CONTENT_ALPHA|1:
+	return "a&";
     case CAIRO_CONTENT_COLOR_ALPHA:
 	return "rgba";
+    case CAIRO_CONTENT_COLOR_ALPHA|1:
+	return "rgba&";
     default:
 	return "<unknown_content>";
     }
 }
 
+static cairo_bool_t
+cairo_perf_has_similar (cairo_perf_t *perf)
+{
+    cairo_surface_t *target = cairo_get_target (perf->cr);
+
+    /* exclude the image backend */
+    if (cairo_surface_get_type (target) == CAIRO_SURFACE_TYPE_IMAGE)
+	return FALSE;
+
+    return TRUE;
+}
+
 void
 cairo_perf_run (cairo_perf_t		*perf,
 		const char		*name,
 		cairo_perf_func_t	 perf_func)
 {
     static cairo_bool_t first_run = TRUE;
-    unsigned int i;
+    unsigned int i, similar, has_similar;
     cairo_perf_ticks_t *times;
     cairo_stats_t stats = {0.0, 0.0};
     int low_std_dev_count;
@@ -135,56 +171,68 @@ cairo_perf_run (cairo_perf_t		*perf,
 
     times = xmalloc (perf->iterations * sizeof (cairo_perf_ticks_t));
 
-    /* We run one iteration in advance to warm caches, etc. */
-    cairo_perf_yield ();
-    (perf_func) (perf->cr, perf->size, perf->size);
-
-    low_std_dev_count = 0;
-    for (i =0; i < perf->iterations; i++) {
+    has_similar = cairo_perf_has_similar (perf);
+    for (similar = 0; similar <= has_similar; similar++) {
+	/* We run one iteration in advance to warm caches, etc. */
 	cairo_perf_yield ();
-	times[i] = (perf_func) (perf->cr, perf->size, perf->size);
+	if (similar)
+	    cairo_push_group_with_content (perf->cr, perf->target->content);
+	(perf_func) (perf->cr, perf->size, perf->size);
+	if (similar)
+	    cairo_pattern_destroy (cairo_pop_group (perf->cr));
 
-	if (perf->raw) {
-	    if (i == 0)
-		printf ("[*] %s-%s %s-%d %g",
-			perf->target->name,
-			_content_to_string (perf->target->content),
-			name, perf->size,
-			cairo_perf_ticks_per_second () / 1000.0);
-	    printf (" %lld", times[i]);
-	} else {
-	    if (i > 0) {
-		_cairo_stats_compute (&stats, times, i+1);
+	low_std_dev_count = 0;
+	for (i =0; i < perf->iterations; i++) {
+	    cairo_perf_yield ();
+	    if (similar)
+		cairo_push_group_with_content (perf->cr, perf->target->content);
+	    times[i] = (perf_func) (perf->cr, perf->size, perf->size);
+	    if (similar)
+		cairo_pattern_destroy (cairo_pop_group (perf->cr));
 
-		if (stats.std_dev <= CAIRO_PERF_LOW_STD_DEV &&
-		    ! perf->exact_iterations)
-		{
-		    low_std_dev_count++;
-		    if (low_std_dev_count >= CAIRO_PERF_STABLE_STD_DEV_COUNT)
-			break;
-		} else {
-		    low_std_dev_count = 0;
+	    if (perf->raw) {
+		if (i == 0)
+		    printf ("[*] %s-%s %s-%d %g",
+			    perf->target->name,
+			    _content_to_string (perf->target->content, similar),
+			    name, perf->size,
+			    cairo_perf_ticks_per_second () / 1000.0);
+		printf (" %lld", times[i]);
+	    } else if (! perf->exact_iterations) {
+		if (i > 0) {
+		    _cairo_stats_compute (&stats, times, i+1);
+
+		    if (stats.std_dev <= CAIRO_PERF_LOW_STD_DEV)
+		    {
+			low_std_dev_count++;
+			if (low_std_dev_count >= CAIRO_PERF_STABLE_STD_DEV_COUNT)
+			    break;
+		    } else {
+			low_std_dev_count = 0;
+		    }
 		}
 	    }
 	}
+
+	if (perf->raw) {
+	    printf ("\n");
+	} else {
+	    _cairo_stats_compute (&stats, times, i);
+	    printf ("[%3d] %8s-%-5s %26s-%-3d ",
+		    perf->test_number, perf->target->name,
+		    _content_to_string (perf->target->content, similar),
+		    name, perf->size);
+
+	    printf ("%10lld %#8.3f %#8.3f %#5.2f%% %3d\n",
+		    stats.min_ticks,
+		    (stats.min_ticks * 1000.0) / cairo_perf_ticks_per_second (),
+		    (stats.median_ticks * 1000.0) / cairo_perf_ticks_per_second (),
+		    stats.std_dev * 100.0, stats.iterations);
+	}
+
+	perf->test_number++;
     }
-
-    if (perf->raw) {
-	printf ("\n");
-    } else {
-	printf ("[%3d] %8s-%-4s %26s-%-3d ",
-		perf->test_number, perf->target->name,
-		_content_to_string (perf->target->content),
-		name, perf->size);
-
-	printf ("%10lld %#8.3f %#8.3f %#5.2f%% %3d\n",
-		stats.min_ticks,
-		(stats.min_ticks * 1000.0) / cairo_perf_ticks_per_second (),
-		(stats.median_ticks * 1000.0) / cairo_perf_ticks_per_second (),
-		stats.std_dev * 100.0, stats.iterations);
-    }
-
-    perf->test_number++;
+    free (times);
 }
 
 static void
@@ -210,10 +258,11 @@ static void
 parse_options (cairo_perf_t *perf, int argc, char *argv[])
 {
     int c;
+    const char *iters;
     char *end;
 
-    if (getenv("CAIRO_PERF_ITERATIONS"))
-	perf->iterations = strtol(getenv("CAIRO_PERF_ITERATIONS"), NULL, 0);
+    if ((iters = getenv("CAIRO_PERF_ITERATIONS")) && *iters)
+	perf->iterations = strtol(iters, NULL, 0);
     else
 	perf->iterations = CAIRO_PERF_ITERATIONS_DEFAULT;
     perf->exact_iterations = 0;
@@ -224,7 +273,7 @@ parse_options (cairo_perf_t *perf, int argc, char *argv[])
     perf->num_names = 0;
 
     while (1) {
-	c = getopt (argc, argv, "i:lr");
+	c = _cairo_getopt (argc, argv, "i:lr");
 	if (c == -1)
 	    break;
 
@@ -258,26 +307,84 @@ parse_options (cairo_perf_t *perf, int argc, char *argv[])
     }
 }
 
+static int 
+check_cpu_affinity(void)
+{
+#ifdef HAVE_SCHED_GETAFFINITY
+
+    cpu_set_t affinity;
+    int i, cpu_count;
+
+    if (sched_getaffinity(0, sizeof(affinity), &affinity)) {
+        perror("sched_getaffinity");
+        return -1;
+    }
+
+    for(i = 0, cpu_count = 0; i < CPU_SETSIZE; ++i) {
+        if (CPU_ISSET(i, &affinity))
+            ++cpu_count;
+    }
+
+    if (cpu_count > 1) {
+        fputs(
+            "WARNING: cairo-perf has not been bound to a single CPU.\n", 
+            stderr);
+        return -1;
+    }
+
+    return 0;
+#else
+    fputs(
+        "WARNING: Cannot check CPU affinity for this platform.\n", 
+        stderr);
+    return -1;
+#endif
+}
+
+static void
+cairo_perf_fini (void)
+{
+    cairo_debug_reset_static_data ();
+#if HAVE_FCFINI
+    FcFini ();
+#endif
+}
+
+
 int
 main (int argc, char *argv[])
 {
-    int i, j;
+    int i, j, num_targets;
     cairo_perf_case_t *perf_case;
     cairo_perf_t perf;
-    const char *cairo_test_target = getenv ("CAIRO_TEST_TARGET");
-    cairo_boilerplate_target_t *target;
+    cairo_boilerplate_target_t **targets;
     cairo_surface_t *surface;
 
     parse_options (&perf, argc, argv);
 
-    for (i = 0; targets[i].name; i++) {
-	perf.target = target = &targets[i];
-	perf.test_number = 0;
+    if (check_cpu_affinity()) {
+        fputs(
+            "NOTICE: cairo-perf and the X server should be bound to CPUs (either the same\n"
+            "or separate) on SMP systems. Not doing so causes random results when the X\n"
+            "server is moved to or from cairo-perf's CPU during the benchmarks:\n"
+            "\n"
+            "    $ sudo taskset -cp 0 $(pidof X)\n"
+            "    $ taskset -cp 1 $$\n"
+            "\n"
+            "See taskset(1) for information about changing CPU affinity.\n",
+            stderr);
+    }
+
+    targets = cairo_boilerplate_get_targets (&num_targets, NULL);
+
+    for (i = 0; i < num_targets; i++) {
+        cairo_boilerplate_target_t *target = targets[i];
 
 	if (! target_is_measurable (target))
 	    continue;
-	if (cairo_test_target && ! strstr (cairo_test_target, target->name))
-	    continue;
+
+	perf.target = target;
+	perf.test_number = 0;
 
 	for (j = 0; perf_cases[j].run; j++) {
 
@@ -292,6 +399,15 @@ main (int argc, char *argv[])
 						    perf.size, perf.size,
 						    CAIRO_BOILERPLATE_MODE_PERF,
 						    &target->closure);
+		if (surface == NULL) {
+		    fprintf (stderr,
+			     "Error: Failed to create target surface: %s\n",
+			     target->name);
+		    cairo_boilerplate_free_targets (targets);
+		    cairo_perf_fini ();
+		    exit (1);
+		}
+
 		cairo_perf_timer_set_synchronize (target->synchronize,
 						  target->closure);
 
@@ -302,20 +418,29 @@ main (int argc, char *argv[])
 		if (cairo_status (perf.cr)) {
 		    fprintf (stderr, "Error: Test left cairo in an error state: %s\n",
 			     cairo_status_to_string (cairo_status (perf.cr)));
+		    cairo_boilerplate_free_targets (targets);
+		    cairo_perf_fini ();
 		    exit (1);
 		}
 
 		cairo_destroy (perf.cr);
 		cairo_surface_destroy (surface);
+
+		if (target->cleanup)
+		    target->cleanup (target->closure);
 	    }
 	}
     }
 
+    cairo_boilerplate_free_targets (targets);
+    cairo_perf_fini ();
+
     return 0;
 }
 
 cairo_perf_case_t perf_cases[] = {
     { paint,  256, 512},
+    { paint_with_alpha,  256, 512},
     { fill,   64, 256},
     { stroke, 64, 256},
     { text,   64, 256},
@@ -329,5 +454,6 @@ cairo_perf_case_t perf_cases[] = {
     { long_lines, 100, 100},
     { unaligned_clip, 100, 100},
     { rectangles, 512, 512},
+    { long_dashed_lines, 512, 512},
     { NULL }
 };
