@@ -27,9 +27,9 @@
 #include "cairo-test.h"
 #include <stdio.h>
 
-#define WIDTH 64
-#define HEIGHT 64
-#define PAD 10
+#define WIDTH 16
+#define HEIGHT 16
+#define PAD 2
 
 const char	png_filename[]	= "romedalen.png";
 
@@ -168,7 +168,7 @@ static cairo_test_draw_function_t draw;
 
 cairo_test_t test = {
     "trap-clip",
-    "Trapezoid clipping",
+    "Trapezoid clipping\n",
     IMAGE_WIDTH, IMAGE_HEIGHT,
     draw
 };
@@ -191,7 +191,7 @@ draw (cairo_t *cr, int width, int height)
 		pattern_funcs[i] (cr, x, y);
 		draw_funcs[j] (cr, x, y);
 		if (cairo_status (cr))
-		    cairo_test_log ("%d %d HERE!\n", i, j);
+		    cairo_test_log ("%d %d HERE!\n", (int)i, (int)j);
 
 		cairo_restore (cr);
 	    }
@@ -199,7 +199,7 @@ draw (cairo_t *cr, int width, int height)
     }
 
     if (cairo_status (cr) != CAIRO_STATUS_SUCCESS)
-	cairo_test_log ("%d %d .HERE!\n", i, j);
+	cairo_test_log ("%d %d .HERE!\n", (int)i, (int)j);
 
     return CAIRO_TEST_SUCCESS;
 }
